﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace NestedLambdasInLinq {

	record OrderItem {
		public string Name { get; init; }
		public int Count { get; init; }
		public string RecipientName { get; init; }
	};

	record Order {
		public DateTime Date { get; init; }
		public List<OrderItem> Items { get; init; }
	}

	record Customer {
		public string Name { get; init; }
		public List<Order> Orders { get; init; }
	}

	class Program {

		static IEnumerable<Customer> GetCustomersWithContaminatedItems(string itemName, DateTime contaminatedBefore) {
			var customersInDanger = from c in Customers
									where c.Orders.Any(
										order => 
											order.Date.CompareTo(contaminatedBefore) < 0
											&& order.Items.Any(item => item.Name == itemName)
									) select c;
			return customersInDanger;
		}

		static void Main(string[] args) {

			var warningRecipients = GetCustomersWithContaminatedItems(itemName: "orange-juice", contaminatedBefore: DateTime.Today.AddDays(-5));

			foreach (var recipient in warningRecipients) {
				Console.WriteLine($"Email to {recipient.Name} sent.");
			}

			Console.WriteLine();
			Console.WriteLine("---");
			Console.WriteLine();

			var allOrders = Customers.SelectMany(c => c.Orders).ToList();
			foreach (var o in allOrders) {
				Console.WriteLine(o);
			}
		}

		static List<Customer> Customers = new() {
			new Customer {
				Name = "Jezek", Orders = new() {
					new Order {
						Date = DateTime.Today.AddDays(-1), Items = new() {
							new OrderItem { Name = "ham", Count = 2, RecipientName = "Jezek" },
							new OrderItem { Name = "cola", Count = 1, RecipientName = "Jezek" },
							new OrderItem { Name = "bun", Count = 5, RecipientName = "Jezek" }
						}
					},
					new Order {
						Date = DateTime.Today.AddDays(-7), Items = new() {
							new OrderItem { Name = "cola", Count = 2, RecipientName = "Jezek" },
							new OrderItem { Name = "bun", Count = 3, RecipientName = "Jezek" }
						}
					},
					new Order {
						Date = DateTime.Today.AddDays(-10), Items = new() {
							new OrderItem { Name = "orange-juice", Count = 3, RecipientName = "Jezek" },
							new OrderItem { Name = "ham", Count = 1, RecipientName = "Jezek" },
							new OrderItem { Name = "bun", Count = 1, RecipientName = "Jezek" }
						}
					}
				}
			},
			new Customer {
				Name = "Adamek", Orders = new() {
					new Order {
						Date = DateTime.Today.AddDays(-1), Items = new() {
							new OrderItem { Name = "ham", Count = 2, RecipientName = "Adamek" },
							new OrderItem { Name = "cola", Count = 1, RecipientName = "Adamek" },
							new OrderItem { Name = "bun", Count = 5, RecipientName = "Adamek" }
						}
					},
					new Order {
						Date = DateTime.Today.AddDays(-7), Items = new() {
							new OrderItem { Name = "cola", Count = 2, RecipientName = "Adamek" },
							new OrderItem { Name = "bun", Count = 3, RecipientName = "Adamek" }
						}
					},
					new Order {
						Date = DateTime.Today.AddDays(-10), Items = new() {
							new OrderItem { Name = "orange-juice", Count = 3, RecipientName = "Vesely" },
							new OrderItem { Name = "ham", Count = 1, RecipientName = "Adamek" },
							new OrderItem { Name = "bun", Count = 1, RecipientName = "Adamek" }
						}
					}
				}
			},
			new Customer {
				Name = "Hnetynka", Orders = new() {
					new Order {
						Date = DateTime.Today.AddDays(-2), Items = new() {
							new OrderItem { Name = "water", Count = 1, RecipientName = "Hnetynka" },
							new OrderItem { Name = "orange-juice", Count = 1, RecipientName = "Hnetynka" },
							new OrderItem { Name = "bun", Count = 2, RecipientName = "Hnetynka" }
						}
					},
					new Order {
						Date = DateTime.Today.AddDays(-4), Items = new() {
							new OrderItem { Name = "water", Count = 1, RecipientName = "Hnetynka" },
							new OrderItem { Name = "orange-juice", Count = 1, RecipientName = "Hnetynka" },
							new OrderItem { Name = "bun", Count = 2, RecipientName = "Hnetynka" }
						}
					},
					new Order {
						Date = DateTime.Today.AddDays(-30), Items = new() {
							new OrderItem { Name = "water", Count = 1, RecipientName = "Hnetynka" },
							new OrderItem { Name = "bun", Count = 2, RecipientName = "Hnetynka" }
						}
					}
				}
			},
			new Customer {
				Name = "Kofron", Orders = new() {
					new Order {
						Date = DateTime.Today.AddDays(-3), Items = new() {
							new OrderItem { Name = "orange-juice", Count = 1, RecipientName = "Kofron" },
							new OrderItem { Name = "bun", Count = 2, RecipientName = "Kofron" },
							new OrderItem { Name = "marmelade", Count = 1, RecipientName = "Kofron" },
						}
					},
					new Order {
						Date = DateTime.Today.AddDays(-6), Items = new() {
							new OrderItem { Name = "apple-juice", Count = 1, RecipientName = "Kofron" },
							new OrderItem { Name = "bun", Count = 2, RecipientName = "Kofron" },
						}
					},
					new Order {
						Date = DateTime.Today.AddDays(-12), Items = new() {
							new OrderItem { Name = "grapefruit-juice", Count = 1, RecipientName = "Kofron" },
							new OrderItem { Name = "orange-juice", Count = 1, RecipientName = "Kofron" },
							new OrderItem { Name = "bun", Count = 2, RecipientName = "Kofron" },
							new OrderItem { Name = "marmelade", Count = 1, RecipientName = "Kofron" },
						}
					}
				}
			},
		};
	}
}
