﻿public class DebugInput {
	public required Route[] ControlledRoutes { get; init; }
	public required PointGroup[] ControlledPointGroups { get; init; }

	public IEnumerator<CoroutineStepResult> GetCoroutine() {
		while (true) {
			if (Console.KeyAvailable) {
				var key = Console.ReadKey(intercept: true);
				switch (key.Key) {
					case ConsoleKey.Q: CoroutineScheduler.Current.Enqueue(new CallRouteDriver(ControlledRoutes[0])); break;
					case ConsoleKey.W: CoroutineScheduler.Current.Enqueue(new CallRouteDriver(ControlledRoutes[1])); break;
					case ConsoleKey.E: CoroutineScheduler.Current.Enqueue(new CallRouteDriver(ControlledRoutes[2])); break;
					case ConsoleKey.R: CoroutineScheduler.Current.Enqueue(new CallRouteDriver(ControlledRoutes[3])); break;
					case ConsoleKey.A: CoroutineScheduler.Current.Enqueue(new CancelRouteDriver(ControlledRoutes[0])); break;
					case ConsoleKey.S: CoroutineScheduler.Current.Enqueue(new CancelRouteDriver(ControlledRoutes[1])); break;
					case ConsoleKey.D: CoroutineScheduler.Current.Enqueue(new CancelRouteDriver(ControlledRoutes[2])); break;
					case ConsoleKey.F: CoroutineScheduler.Current.Enqueue(new CancelRouteDriver(ControlledRoutes[3])); break;

					case ConsoleKey.D1: CoroutineScheduler.Current.Enqueue(new SetPointGroupPositionDriver(ControlledPointGroups[0], PointPosition.Normal)); break;
					case ConsoleKey.D2: CoroutineScheduler.Current.Enqueue(new SetPointGroupPositionDriver(ControlledPointGroups[0], PointPosition.Reverse)); break;
					case ConsoleKey.D3: CoroutineScheduler.Current.Enqueue(new SetPointGroupPositionDriver(ControlledPointGroups[1], PointPosition.Normal)); break;
					case ConsoleKey.D4: CoroutineScheduler.Current.Enqueue(new SetPointGroupPositionDriver(ControlledPointGroups[1], PointPosition.Reverse)); break;
					case ConsoleKey.D5: CoroutineScheduler.Current.Enqueue(new SetPointGroupPositionDriver(ControlledPointGroups[2], PointPosition.Normal)); break;
					case ConsoleKey.D6: CoroutineScheduler.Current.Enqueue(new SetPointGroupPositionDriver(ControlledPointGroups[2], PointPosition.Reverse)); break;
				}
			}
			yield return CoroutineStepResult.Success();
		}
	}
}

public class DebugStateDumper {
	public required Point[] Points { get; init; }
	public required TvpSection?[] TvpSections { get; init; }
	public required CoroutineScheduler Scheduler { get; init; }
	public required MockFieldObjectMessenger Messenger { get; init; }

	private static char[] s_DriverAnimationChars = { 'I', '/', '-', '\\' };

	public IEnumerator<CoroutineStepResult> GetPeriodicDumpingCoroutine() {
		Console.Clear();

		while (true) {
			Console.SetCursorPosition(0, 0);

			// Print points:
			Console.WriteLine(">>> Points >>>");
			foreach (var point in Points) {
				Console.WriteLine($"  {(point.IsRouteLocked ? 'X' : '.')} {point.Position.ToSymbolChar()} {point.Name}");
			}

			// Print TVP sections:
			Console.WriteLine(">>> TVP sections >>>");
			foreach (var section in TvpSections) {
				if (section is null) {
					Console.WriteLine();
				} else {
					Console.Write($"\t[{section.Name} {(section.IsRouteLocked ? 'X' : '.')} {(section.IsVacant ? 'v' : 'O')}]");
				}
			}

			// Print coroutines nad drivers:
			Console.WriteLine($">>> Coroutine scheduler - current time: {Scheduler.CurrentTime} ms >>>");
			for (CoroutinePriority priority = CoroutinePriority.PreDriver; priority <= CoroutinePriority.Last; priority++) {
				if (priority == CoroutinePriority.Driver) {
					for (int i = 0; i < Scheduler.Drivers.Length; i++) {
						Console.Write($"  Driver {i}: ");
						var driver = Scheduler.Drivers[i];
						if (driver is null) {
							Console.WriteLine("---                          ");
						} else {
							if (driver is CallRouteDriver callRouteDriver) {
								Console.Write($"VC {callRouteDriver.Route.EntrySignal?.Name} [{callRouteDriver.Route.Id}]");
							} else if (driver is CancelRouteDriver cancelRouteDriver) {
								Console.Write($"RC {cancelRouteDriver.Route.EntrySignal?.Name} [{cancelRouteDriver.Route.Id}]");
							} else if (driver is SetPointGroupPositionDriver setPositionDriver) {
								Console.Write($"S{setPositionDriver.TargetPosition.ToSymbolChar()} {setPositionDriver.PointGroup.Name}");
							}

							int animationFrame = ((Scheduler.CurrentTime - driver.StartTime) / 200) % s_DriverAnimationChars.Length;
                            Console.WriteLine($" {s_DriverAnimationChars[animationFrame]}");
                        }
					}
				} else {
					Console.WriteLine($"  {priority}: {Scheduler.CoroutineQueues[(int) priority].Count} coroutines");
				}
			}

			// Print errors:
			for (int i = 0; i < Scheduler.Errors.Count; i++) {
				Console.SetCursorPosition(20, i);
				Console.WriteLine($"Error {i}: {Scheduler.Errors[Scheduler.Errors.Count - i - 1]}                                  ");
			}

			// Print drives:
			for (int i = 0; i < Messenger.PointDrives.Length; i++) {
				Console.SetCursorPosition(60, i);
				var drive = Messenger.PointDrives[i];
				Console.WriteLine($"{(drive.IsMotorRunning ? 'R' : '-')} -> {drive.TargetPosition.ToSymbolChar()} drive {drive.Name}");
			}

			yield return CoroutineStepResult.Success();
		}
	}
}