﻿public abstract class FieldObjectMessenger {
	public abstract void ThrowPoint(Point point, PointPosition targetPosition);
}

public class MockPointDrive : PointDrive {
	public PointPosition TargetPosition { get; set; }
	public bool IsMotorRunning { get; set; } = false;
	public TimeoutGuard CompleteThrowTimeout { get; set; }

	public void StartMotor(PointPosition targetPosition) {
		if (targetPosition != TargetPosition) {
			CompleteThrowTimeout = new TimeoutGuard(2_500);
		}
		TargetPosition = targetPosition;
		IsMotorRunning = true;
	}

	public void StopMotor() {
		IsMotorRunning = false;
	}
}

public class MockFieldObjectMessenger : FieldObjectMessenger {
	private MockPointDrive[] _pointDrives;

	public MockFieldObjectMessenger(MockPointDrive[] pointDrives) {
		_pointDrives = pointDrives;
	}

	public MockPointDrive[] PointDrives => _pointDrives;

	public override void ThrowPoint(Point point, PointPosition targetPosition) {
		point.Position = PointPosition.None;

		// TODO: Implement starting drive motors in sequence:
		//       Start just motor of the 1st drive -> motor of 2nd drive should be started by GetMotorRunningCoroutine() after 1st drive is in targetPosition.
		// TODO: Do not start motors of drives already in targetPosition.
		foreach (MockPointDrive drive in point.Drives) {
			drive.StartMotor(targetPosition);
		}
	}

	public IEnumerator<CoroutineStepResult> GetMotorRunningCoroutine() {
		while (true) {
			foreach (var drive in _pointDrives) {
				if (drive.IsMotorRunning && drive.CompleteThrowTimeout.HasExpired) {
					drive.StopMotor();

					if (drive.Point.Drives.Cast<MockPointDrive>().All(d => !d.IsMotorRunning && d.TargetPosition == drive.TargetPosition)) {
						drive.Point.Position = drive.TargetPosition;
					}
				}

			}
			yield return CoroutineStepResult.Success();
		}
	}
}